#ifndef __SUNINTERVAL_VECTOR_H
#define __SUNINTERVAL_VECTOR_H

#include <valarray>
#include <iostream.h>
#include "suninterval_valarray.h"

namespace SUNW_interval {

//
// nvector
//
template< typename T >
class nvector {
    struct vrep;
    vrep *rep;
public:
    class vref;
    nvector(size_t size);
    nvector(const T& val, size_t size);
    nvector(const T* pointer, size_t size);
    nvector(const std::valarray<T>& vec);
    nvector(const nvector<T>& vec);
    nvector<T>& operator= (const nvector<T>& vec);
    nvector<T>& operator= (const T& val);
    ~nvector();

    T operator[] (size_t i) const { return rep->v[i]; }
    vref operator[] (size_t i) { return vref(*this, i); }

    nvector<T> operator+() const { return *this; }
    nvector<T> operator-() const { return -rep->v; }

    nvector<T>& operator+= (const nvector<T>& vec);
    nvector<T>& operator-= (const nvector<T>& vec);
    nvector<T>& operator*= (const nvector<T>& vec);
    nvector<T>& operator/= (const nvector<T>& vec);
    nvector<T>& operator+= (const T& val);
    nvector<T>& operator-= (const T& val);
    nvector<T>& operator*= (const T& val);
    nvector<T>& operator/= (const T& val);
        
    size_t size() const { return rep->size(); }

    T& ref(size_t i);
    T& direct_ref(size_t i) { return rep->v[i]; };
    std::valarray<T>& array() const { return rep->v; }
};

//
// nvector representation class
//
template< typename T >
struct nvector<T>::vrep {
    std::valarray<T> v;
    unsigned n;
    vrep(size_t size);
    vrep(const T& val, size_t size);
    vrep(const T* pointer, size_t size);
    vrep(const std::valarray<T>& vec);

    vrep *get_own_copy() {
        if (n == 1) 
            return this;
        n--;
        return new vrep(v);
    }
    
    size_t size() const { return v.size(); }
};

//
// reference to nvector element
//
template< typename T >
class nvector<T>::vref {
    nvector<T>& vec;
    size_t i;
public:
    vref(nvector<T>& vvec, size_t ii) : vec(vvec), i(ii) { }
    operator T () const { return vec.array()[i]; };
    void operator= (const T& x) {
        vec.ref(i) = x;
    }
    void operator= (const vref& r) {
        vec.ref(i) = (T)r;
    }
    void operator+= (const T& x) {
        vec.ref(i) += x;
    }
    void operator-= (const T& x) {
        vec.ref(i) -= x;
    }
    void operator*= (const T& x) {
        vec.ref(i) *= x;
    }
    void operator/= (const T& x) {
        vec.ref(i) /= x;
    }
};

template< typename T >
inline nvector<T>::vrep::vrep(size_t size) : v(size), n(1) { }

template< typename T >
inline nvector<T>::vrep::vrep(const T& val, size_t size) : v(val, size), n(1) { }

template< typename T >
inline nvector<T>::vrep::vrep(const T* pointer, size_t size) : v(pointer, size), n(1) { }

template< typename T >
inline nvector<T>::vrep::vrep(const std::valarray<T>& vec) : v(vec), n(1) { }

template< typename T >
inline T& nvector<T>::ref(size_t i)
{
    rep = rep->get_own_copy();
    return rep->v[i];
}
    
template< typename T >
inline nvector<T>::nvector(size_t size) 
{
    rep = new vrep(size);
}

template< typename T >
inline nvector<T>::nvector(const T& val, size_t size) 
{
    rep = new vrep(val, size);
}

template< typename T >
inline nvector<T>::nvector(const T* pointer, size_t size) 
{
    rep = new vrep(pointer, size);
}

template< typename T >
inline nvector<T>::nvector(const std::valarray<T>& vec) 
{
    rep = new vrep(vec);
}

template< typename T >
inline nvector<T>::nvector(const nvector<T>& vec) 
{
    vrep *t = vec.rep;
    t->n++;
    rep = t;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator= (const nvector<T>& vec) 
{
    vrep *t = vec.rep;
    t->n++;
    if (--rep->n == 0)
        delete rep;
    rep = t;
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator= (const T& val) 
{
    rep = rep->get_own_copy();
    rep->v = val;
    return *this;
}

template< typename T >
inline nvector<T>::~nvector() 
{
    if (--rep->n == 0) 
        delete rep;
}
  
template< typename T >
inline nvector<T>& 
nvector<T>::operator+= (const nvector<T>& vec) 
{
    rep = rep->get_own_copy();
    rep->v += vec.array();
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator-= (const nvector<T>& vec) 
{ 
    rep = rep->get_own_copy();
    rep->v -= vec.array();
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator*= (const nvector<T>& vec) 
{ 
    rep = rep->get_own_copy();
    rep->v *= vec.array();
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator/= (const nvector<T>& vec) 
{ 
    rep = rep->get_own_copy();
    rep->v /= vec.array();
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator+= (const T& val) 
{
    rep = rep->get_own_copy();
    rep->v += val;
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator-= (const T& val) 
{ 
    rep = rep->get_own_copy();
    rep->v -= val;
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator*= (const T& val) 
{ 
    rep = rep->get_own_copy();
    rep->v *= val;
    return *this;
}

template< typename T >
inline nvector<T>& 
nvector<T>::operator/= (const T& val) 
{ 
    rep = rep->get_own_copy();
    rep->v /= val;
    return *this;
}

// nonmember operators and functions with nvector

template< typename T >
inline nvector<T> 
operator+ (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() + b.array();
}

template< typename T >
inline nvector<T> 
operator- (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() - b.array();
}

template< typename T >
inline nvector<T> 
operator* (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() * b.array();
}

template< typename T >
inline nvector<T> 
operator/ (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() / b.array();
}

template< typename T >
inline nvector<bool>
operator== (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() == b.array();
}

template< typename T >
inline nvector<bool>
operator!= (const nvector<T>& a, const nvector<T>& b)
{
    return a.array() != b.array();
}

template< typename T >
inline nvector<T> 
operator+ (const nvector<T>& a, const T& b)
{
    return a.array() + b;
}

template< typename T >
inline nvector<T> 
operator- (const nvector<T>& a, const T& b)
{
    return a.array() - b;
}

template< typename T >
inline nvector<T> 
operator* (const nvector<T>& a, const T& b)
{
    return a.array() * b;
}

template< typename T >
inline nvector<T> 
operator/ (const nvector<T>& a, const T& b)
{
    return a.array() / b;
}

template< typename T >
inline nvector<bool>
operator== (const nvector<T>& a, const T& b)
{
    return a.array() == b;
}

template< typename T >
inline nvector<bool>
operator!= (const nvector<T>& a, const T& b)
{
    return a.array() != b;
}

template< typename T >
inline nvector<T> 
operator+ (const T& a, const nvector<T>& b)
{
    return a + b.array();
}

template< typename T >
inline nvector<T> 
operator- (const T& a, const nvector<T>& b)
{
    return a - b.array();
}

template< typename T >
inline nvector<T> 
operator* (const T& a, const nvector<T>& b)
{
    return a * b.array();
}

template< typename T >
inline nvector<T> 
operator/ (const T& a, const nvector<T>& b)
{
    return a / b.array();
}

template< typename T >
inline nvector<bool>
operator== (const T& a, const nvector<T>& b)
{
    return a == b.array();
}

template< typename T >
inline nvector<bool>
operator!= (const T& a, const nvector<T>& b)
{
    return a != b.array();
}

/* Extraction Functions: infimim, supremum, midpoint, width, magnitude,
mignitude */

template< typename T >
inline nvector<T> inf(const nvector< interval<T> >& a)
{
    return inf(a.array());
}

template< typename T >
inline nvector<T> sup(const nvector< interval<T> >& a)
{
    return sup(a.array());
}

template< typename T >
inline nvector<T> mid(const nvector< interval<T> >& a)
{
    return mid(a.array());
}

template< typename T >
inline nvector<T> wid(const nvector< interval<T> >& a)
{
    return wid(a.array());
}

template< typename T >
inline nvector<T> mag(const nvector< interval<T> >& a)
{
    return mag(a.array());
}

template< typename T >
inline nvector<T> mig(const nvector< interval<T> >& a)
{
    return mig(a.array());
}

/* Intersection Division */

template< typename T >
inline nvector<T>
divix(const nvector<T>& a, const nvector<T>& b, const nvector<T>& c)
{
    return divix(a.array(), b.array(), c.array());
}

/* Hull Functions: regular hull, intersection */

template< typename T >
inline nvector<T>
interval_hull(const nvector<T>& a, const nvector<T>& b)
{
    return interval_hull(a.array(), b.array());
}

template< typename T >
inline nvector<T>
intersect(const nvector<T>& a, const nvector<T>& b)
{
    return intersect(a.array(), b.array());
}

template< typename T >
inline nvector<T>
interval_hull(const nvector<T>& a, const T& b)
{
    return interval_hull(a.array(), b);
}

template< typename T >
inline nvector<T>
intersect(const nvector<T>& a, const T& b)
{
    return intersect(a.array(), b);
}

template< typename T >
inline nvector<T>
interval_hull(const T& a, const nvector<T>& b)
{
    return interval_hull(a, b.array());
}

template< typename T >
inline nvector<T>
intersect(const T& a, const nvector<T>& b)
{
    return intersect(a, b.array());
}

/* Query Functions: is empty, ndigits */

template< typename T >
inline nvector<interval_bool> is_empty(const nvector<T>& a)
{
    return is_empty(a.array());
}

template< typename T >
inline nvector<int> ndigits(const nvector<T>& a)
{
    return ndigits(a.array());
}

/* Membership Functions */

template< typename T1, typename T2 >
inline nvector<interval_bool>
in(const nvector<T1>& a, const nvector<T2>& b)
{
    return in(a.array(), b.array());
}

/* Comparison Functions: disjoint, in interior, subset, superset,
proper subset, proper superset */

template< typename T >
inline nvector<interval_bool>
disjoint(const nvector<T>& a, const nvector<T>& b)
{
    return disjoint(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
in_interior(const nvector<T>& a, const nvector<T>& b)
{
    return in_interior(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
subset(const nvector<T>& a, const nvector<T>& b)
{
    return subset(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
superset(const nvector<T>& a, const nvector<T>& b)
{
    return superset(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
proper_subset(const nvector<T>& a, const nvector<T>& b)
{
    return proper_subset(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
proper_superset(const nvector<T>& a, const nvector<T>& b)
{
    return proper_superset(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
disjoint(const nvector<T>& a, const T& b)
{
    return disjoint(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
in_interior(const nvector<T>& a, const T& b)
{
    return in_interior(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
subset(const nvector<T>& a, const T& b)
{
    return subset(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
superset(const nvector<T>& a, const T& b)
{
    return superset(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
proper_subset(const nvector<T>& a, const T& b)
{
    return proper_subset(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
proper_superset(const nvector<T>& a, const T& b)
{
    return proper_superset(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
disjoint(const T& a, const nvector<T>& b)
{
    return disjoint(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
in_interior(const T& a, const nvector<T>& b)
{
    return in_interior(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
subset(const T& a, const nvector<T>& b)
{
    return subset(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
superset(const T& a, const nvector<T>& b)
{
    return superset(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
proper_subset(const T& a, const nvector<T>& b)
{
    return proper_subset(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
proper_superset(const T& a, const nvector<T>& b)
{
    return proper_superset(a, b.array());
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
seq(const nvector<T>& a, const nvector<T>& b)
{
    return seq(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
sne(const nvector<T>& a, const nvector<T>& b)
{
    return sne(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
slt(const nvector<T>& a, const nvector<T>& b)
{
    return slt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
sle(const nvector<T>& a, const nvector<T>& b)
{
    return sle(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
sgt(const nvector<T>& a, const nvector<T>& b)
{
    return sgt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
sge(const nvector<T>& a, const nvector<T>& b)
{
    return sge(a.array(), b.array());
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
ceq(const nvector<T>& a, const nvector<T>& b)
{
    return ceq(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
cne(const nvector<T>& a, const nvector<T>& b)
{
    return cne(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
clt(const nvector<T>& a, const nvector<T>& b)
{
    return clt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
cle(const nvector<T>& a, const nvector<T>& b)
{
    return cle(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
cgt(const nvector<T>& a, const nvector<T>& b)
{
    return cgt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
cge(const nvector<T>& a, const nvector<T>& b)
{
    return cge(a.array(), b.array());
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
peq(const nvector<T>& a, const nvector<T>& b)
{
    return peq(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
pne(const nvector<T>& a, const nvector<T>& b)
{
    return pne(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
plt(const nvector<T>& a, const nvector<T>& b)
{
    return plt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
ple(const nvector<T>& a, const nvector<T>& b)
{
    return ple(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
pgt(const nvector<T>& a, const nvector<T>& b)
{
    return pgt(a.array(), b.array());
}

template< typename T >
inline nvector<interval_bool>
pge(const nvector<T>& a, const nvector<T>& b)
{
    return pge(a.array(), b.array());
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
seq(const nvector<T>& a, const T& b)
{
    return seq(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
sne(const nvector<T>& a, const T& b)
{
    return sne(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
slt(const nvector<T>& a, const T& b)
{
    return slt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
sle(const nvector<T>& a, const T& b)
{
    return sle(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
sgt(const nvector<T>& a, const T& b)
{
    return sgt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
sge(const nvector<T>& a, const T& b)
{
    return sge(a.array(), b);
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
ceq(const nvector<T>& a, const T& b)
{
    return ceq(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
cne(const nvector<T>& a, const T& b)
{
    return cne(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
clt(const nvector<T>& a, const T& b)
{
    return clt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
cle(const nvector<T>& a, const T& b)
{
    return cle(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
cgt(const nvector<T>& a, const T& b)
{
    return cgt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
cge(const nvector<T>& a, const T& b)
{
    return cge(a.array(), b);
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
peq(const nvector<T>& a, const T& b)
{
    return peq(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
pne(const nvector<T>& a, const T& b)
{
    return pne(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
plt(const nvector<T>& a, const T& b)
{
    return plt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
ple(const nvector<T>& a, const T& b)
{
    return ple(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
pgt(const nvector<T>& a, const T& b)
{
    return pgt(a.array(), b);
}

template< typename T >
inline nvector<interval_bool>
pge(const nvector<T>& a, const T& b)
{
    return pge(a.array(), b);
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
seq(const T& a, const nvector<T>& b)
{
    return seq(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
sne(const T& a, const nvector<T>& b)
{
    return sne(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
slt(const T& a, const nvector<T>& b)
{
    return slt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
sle(const T& a, const nvector<T>& b)
{
    return sle(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
sgt(const T& a, const nvector<T>& b)
{
    return sgt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
sge(const T& a, const nvector<T>& b)
{
    return sge(a, b.array());
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
ceq(const T& a, const nvector<T>& b)
{
    return ceq(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
cne(const T& a, const nvector<T>& b)
{
    return cne(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
clt(const T& a, const nvector<T>& b)
{
    return clt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
cle(const T& a, const nvector<T>& b)
{
    return cle(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
cgt(const T& a, const nvector<T>& b)
{
    return cgt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
cge(const T& a, const nvector<T>& b)
{
    return cge(a, b.array());
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

template< typename T >
inline nvector<interval_bool>
peq(const T& a, const nvector<T>& b)
{
    return peq(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
pne(const T& a, const nvector<T>& b)
{
    return pne(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
plt(const T& a, const nvector<T>& b)
{
    return plt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
ple(const T& a, const nvector<T>& b)
{
    return ple(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
pgt(const T& a, const nvector<T>& b)
{
    return pgt(a, b.array());
}

template< typename T >
inline nvector<interval_bool>
pge(const T& a, const nvector<T>& b)
{
    return pge(a, b.array());
}

/* Arithmetic Functions: absolute value, min, max, ceil, floor, fmod */
  
template< typename T >
inline nvector<T> fabs(const nvector<T>& vec)
{
    return fabs(vec.array());
}

template< typename T >
inline nvector<T>
minimum(const nvector<T>& a, const nvector<T>& b)
{
    return minimum(a.array(), b.array());
}

template< typename T >
inline nvector<T>
maximum(const nvector<T>& a, const nvector<T>& b)
{
    return maximum(a.array(), b.array());
}

template< typename T >
inline nvector<T>
minimum(const nvector<T>& a, const T& b)
{
    return minimum(a.array(), b);
}

template< typename T >
inline nvector<T>
maximum(const nvector<T>& a, const T& b)
{
    return maximum(a.array(), b);
}

template< typename T >
inline nvector<T>
minimum(const T& a, const nvector<T>& b)
{
    return minimum(a, b.array());
}

template< typename T >
inline nvector<T>
maximum(const T& a, const nvector<T>& b)
{
    return maximum(a, b.array());
}

template< typename T >
inline nvector<T> ceil(const nvector< interval<T> >& a)
{
    return ceil(a.array());
}

template< typename T >
inline nvector<T> floor(const nvector< interval<T> >& a)
{
    return floor(a.array());
}

template< typename T >
inline nvector<T>
fmod(const nvector<T>& a, const nvector<T>& b)
{
    return fmod(a.array(), b.array());
}

template< typename T >
inline nvector<T>
fmod(const nvector<T>& a, const T& b)
{
    return fmod(a.array(), b);
}

template< typename T >
inline nvector<T>
fmod(const T& a, const nvector<T>& b)
{
    return fmod(a, b.array());
}

/* Power Functions: square, square root, cube root, natural power,
natural log, log base 10, integral power, floating power */

template< typename T >
inline nvector<T> sqr(const nvector<T>& vec)
{
    return sqr(vec.array());
}

template< typename T >
inline nvector<T> sqrt(const nvector<T>& vec)
{
    return sqrt(vec.array());
}

template< typename T >
inline nvector<T> cbrt(const nvector<T>& vec)
{
    return cbrt(vec.array());
}

template< typename T >
inline nvector<T> exp(const nvector<T>& vec)
{
    return exp(vec.array());
}

template< typename T >
inline nvector<T> log(const nvector<T>& vec)
{
    return log(vec.array());
}

template< typename T >
inline nvector<T> log10(const nvector<T>& vec)
{
    return log10(vec.array());
}

template< typename T >
inline nvector<T> 
pow(const nvector<T>& a, const nvector<T>& b)
{
    return pow(a.array(), b.array());
}

template< typename T >
inline nvector<T> 
pow(const nvector<T>& a, const T& b)
{
    return pow(a.array(), b);
}

template< typename T >
inline nvector<T> 
pow(const T& a, const nvector<T>& b)
{
    return pow(a, b.array());
}

template< typename T >
inline nvector<T> 
pow(const nvector<T>& a, const nvector<int>& b)
{
    return pow(a.array(), b.array());
}

template< typename T >
inline nvector<T> 
pow(const nvector<T>& a, const int& b)
{
    return pow(a.array(), b);
}

template< typename T >
inline nvector<T> 
pow(const T& a, const nvector<int>& b)
{
    return pow(a, b.array());
}

/* Trigonometric Functions */

template< typename T >
inline nvector<T> acos(const nvector<T>& vec)
{
    return acos(vec.array());
}

template< typename T >
inline nvector<T> asin(const nvector<T>& vec)
{
    return asin(vec.array());
}

template< typename T >
inline nvector<T> atan(const nvector<T>& vec)
{
    return atan(vec.array());
}

template< typename T >
inline nvector<T> cos(const nvector<T>& vec)
{
    return cos(vec.array());
}

template< typename T >
inline nvector<T> cosh(const nvector<T>& vec)
{
    return cosh(vec.array());
}

template< typename T >
inline nvector<T> sin(const nvector<T>& vec)
{
    return sin(vec.array());
}

template< typename T >
inline nvector<T> sinh(const nvector<T>& vec)
{
    return sinh(vec.array());
}

template< typename T >
inline nvector<T> tan(const nvector<T>& vec)
{
    return tan(vec.array());
}

template< typename T >
inline nvector<T> tanh(const nvector<T>& vec)
{
    return tanh(vec.array());
}

template< typename T >
inline nvector<T> 
atan2(const nvector<T>& a, const nvector<T>& b)
{
    return atan2(a.array(), b.array());
}

template< typename T >
inline nvector<T> 
atan2(const nvector<T>& a, const T& b)
{
    return atan2(a.array(), b);
}

template< typename T >
inline nvector<T> 
atan2(const T& a, const nvector<T>& b)
{
    return atan2(a, b.array());
}

template< typename T >
inline T sum(const nvector<T>& vec)
{
    return sum(vec.array());
}

template< typename T >
inline T product(const nvector<T>& vec)
{
    return product(vec.array());
}

template< typename T >
T dot_product(const nvector<T>& a, const nvector<T>& b);

template< typename T >
interval<T> dot_product(const nvector< interval<T> >& a, 
                        const nvector< interval<T> >& b);

template< typename T >
T dot_product(const T *a, const T *b, size_t sz);

template< typename T >
interval<T> dot_product(const interval<T> *a, const interval<T> *b, size_t sz);

template< typename T >
ostream& operator<< (ostream&, const nvector<T>& vec);

}

#endif
